void static
assign_mult_Qee(const float_gpu   &mkappa2,
                      cu_wqf_eo       *dye, 
                      cu_wqf_eo        *ye, 
                     const cu_gvf       *u, 
                     const cu_clvf *fclinv = NULL) {
//
// Multiply eopreconditioned Wilson-Clover-Dirac operator squrared.
//
//  dye = Qee ye
//        v       v
//  Qee = Dee^dag Dee
//  v
//  Dee = 1 - kappa^2 Fee Meo Foo Moe
//
  cu_wqf_eo *we = (cu_wqf_eo *)_mm_malloc(sizeof(cu_wqf_eo),16); 
  int ieo = (*ye).ieo;
  (*we).ipeo = (*ye).ipeo;
  (*we).ieo  = ieo;
#pragma omp parallel for
  for (int igx=0;igx<_NGX;igx++) {
    switch_gpu_(&igx);
    CUDA_SAFE_CALL(cudaMalloc((void**)&((*we).g[igx]),sizeof(cu_wqf_eo_blk)));
    new_q((*we).g[igx],ipeo_d.g[igx],ieo_d[ieo].g[igx],igx_d.g[igx],ipsite_d.g[igx]);
  }

  if ( NULL == fclinv ) {
    assign_mult_eoprec_wd_eo(    mkappa2,  we, ye, u);
    assign_mult_eoprec_dag_wd_eo(mkappa2, dye, we, u);
  } else {
    assign_mult_eoprec_wd_eo(    mkappa2,  we, ye, u, fclinv);
    assign_mult_eoprec_dag_wd_eo(mkappa2, dye, we, u, fclinv);
  }

#pragma omp parallel for
  for (int igx=0;igx<_NGX;igx++) {
    switch_gpu_(&igx);
    CUDA_SAFE_CALL(cudaFree((*we).g[igx]))
  }
  _mm_free(we);

}

void static
assign_mult_Qee_shift(const float_gpu   &mkappa2,
                      const float_gpu   &shift,
                      cu_wqf_eo         *dye, 
                      cu_wqf_eo         *ye, 
                      const cu_gvf       *u, 
                      const cu_clvf *fclinv = NULL) {
//
// Multiply shifted-eopreconditioned Wilson-Clover-Dirac operator squrared.
//
//  dye = ( Qee - shift ) ye
//        v       v
//  Qee = Dee^dag Dee
//  v
//  Dee = 1 - kappa^2 Fee Meo Foo Moe
//

  //
  // dye = Qee ye
  //
  if ( NULL == fclinv ) {
    assign_mult_Qee(mkappa2, dye, ye, u);
  } else {
    assign_mult_Qee(mkappa2, dye, ye, u, fclinv);
  }

  //
  // dye = dye - ye * shift
  //
  accum_sub_mult(dye, shift, ye);

}
