#include "config.h"
module lattice_class
!
!= Lattice Class (defines constans, Lattics size, PU size, Color, Dimension etc.)
!
!== Version
!
! $Id: lattice_class.F90,v 1.23 2011/10/30 02:48:26 ishikawa Exp $
!
  use constants_module
  use random_class
  use counter_class
  implicit none
  private 

  !==================================
  ! constants from constants_module
  !==================================
  public :: DP,Z0,Z1,ZI
  public :: SP,C0,C1,CI
  public :: PI,ZERO,ONE,TWO
  public :: CHARLEN,SW_ON,SW_OFF

  public :: lattice_world
  public :: is_initialized
  public :: new
  public :: delete
  public :: print

  !====================
  ! Lattice constants
  !====================
  public :: NTX,NTY,NTZ,NTT
  public :: NDIMX,NDIMY,NDIMZ
  public :: NDIM
  public :: NX,NY,NZ,NT
  public :: NX1,NY1,NZ1,NT1
  public :: NTH
  public :: NPU
  public :: COL,SPIN,CLSP,CLSPH

  public :: isx, isy, isz, ist
  public :: imunu
  public :: nodeid,ipeo
  public :: ipsite,nodeidup,nodeiddn

  !================================
  ! random number generator
  ! from random_class
  !================================
  public :: rand_gfsr_obj, g_rand
  public :: get, get_rand2, get_rand3, save, read
  type(rand_gfsr_obj), save :: g_rand  ! random number generator, GLOBAL variable.

  logical, save :: m_is_initialized = .FALSE.  ! .FALSE.; lattice is not ready, .TRUE. lattice is ready to use.

  type lattice_world
  !
  ! lattice object
  !
    private
    integer :: idummy(2) = 0
  end type

!********************************
!* set lattice size
!********************************
  integer, parameter :: NTX=_NTX   !  X extent
  integer, parameter :: NTY=_NTY   !  Y extent
  integer, parameter :: NTZ=_NTZ   !  Z extent
  integer, parameter :: NTT=_NTT   !  T extent
!********************************
!* set PU array size
!********************************
  integer, parameter :: NDIMX=_NDIMX  ! X node size
  integer, parameter :: NDIMY=_NDIMY  ! Y node size
  integer, parameter :: NDIMZ=_NDIMZ  ! Z node size

!********************************
  integer, parameter :: NDIM=4        ! lattice dimension
  integer, parameter :: NX=NTX/NDIMX  ! X size/node
  integer, parameter :: NY=NTY/NDIMY  ! Y size/node
  integer, parameter :: NZ=NTZ/NDIMZ  ! Z size/node
  integer, parameter :: NT=NTT        ! T size/node
  integer, parameter :: NX1=NX+1
  integer, parameter :: NY1=NY+1
  integer, parameter :: NZ1=NZ+1
  integer, parameter :: NT1=NT+1
  integer, parameter :: NTH=NT/2
  integer, parameter :: NPU=NDIMX*NDIMY*NDIMZ

  integer, parameter :: COL=3
  integer, parameter :: SPIN=4
  integer, parameter :: CLSP=COL*SPIN
  integer, parameter :: CLSPH=(CLSP/2+1)*(CLSP/4)

  !
  ! isx,isy,isz,ist : nearest neighbor site index
  ! imunu           : hyper-plane index (mu,nu)
  !
  integer, parameter :: isx(NDIM)=(/1,0,0,0/)
  integer, parameter :: isy(NDIM)=(/0,1,0,0/)
  integer, parameter :: isz(NDIM)=(/0,0,1,0/)
  integer, parameter :: ist(NDIM)=(/0,0,0,1/)
  integer, parameter :: imunu(NDIM,NDIM)=RESHAPE((/ 0,-1,-2,-3, &
 &                                                  1, 0,-4,-5, &
 &                                                  2, 4, 0,-6, &
 &                                                  3, 5, 6, 0 /),(/4,4/))

!-------------------------------------------------------------------------------
!      nodeid : node number on whith this program is running, should be 0..NPE-1.
!        ipeo : indicates that this node begins with even/odd physical site.
!   ipsite(i) : node number in i-th direction,  should be ipsize(i)=0..NDIMi,
!               where i = 1,2,3 means i = x,y,z
! nodeidup(i) : indicates next upward node number in i-th direction.
! nodeiddn(i) : indicates next downward node number in i-th direction.
!-------------------------------------------------------------------------------
  integer, save :: nodeid          ! node index(MPI rank), [0..NPE-1].
  integer, save :: ipeo            ! node fiest site even/odd-ness.
  integer, save :: ipsite(NDIM-1)  ! XYZ node location 
  integer, save :: nodeidup(NDIM-1)  ! upper node index
  integer, save :: nodeiddn(NDIM-1)  ! lower node index


  interface new
    !
    ! initialize lattice world
    !
    module procedure new_lattice
  end interface

  interface delete
    !
    ! delete lattice world
    !
    module procedure delete_lattice
  end interface

  interface print
    !
    ! print lattice stracture
    !
    module procedure print_lattice
  end interface

  interface is_initialized
    !
    ! return initialization status
    !
    module procedure is_initialized_lattice
  end interface

contains

subroutine new_lattice(this)
!
! Initialize lattice constants, node information (next to nearest node rank etc.)
!
  use comlib
#ifdef _QMX_
  use qmxf03_module
#endif
  implicit none
  type(lattice_world), intent(inout) :: this
#ifndef _singlePU
  integer :: npe
#endif

  if (m_is_initialized) then
    write(*,'("Lattice_world is already initialized.")')
    return
  endif

!-------------------------
!  Set node information
!-------------------------
#ifndef _singlePU
  call comlib_init
  call comlib_node(nodeid,npe)
  if (npe.NE.NPU) then
    write(*,'(" Error : PE num: NPU=",I5," npe=",I5)')
    call comlib_finalize
    stop 999
  endif

#ifdef _QMX_
  ipsite(1) = qmx_get_2d_local_rank_x()
  ipsite(2) = qmx_get_2d_local_rank_y()
  ipsite(3) = 0

  ipeo=mod(ipsite(1)*NX+ipsite(2)*NY+ipsite(3)*NZ,2)

  nodeidup(1) = qmx_get_2d_next_x_rank()
  nodeiddn(1) = qmx_get_2d_prev_x_rank()
  nodeidup(2) = qmx_get_2d_next_y_rank()
  nodeiddn(2) = qmx_get_2d_prev_y_rank()
  nodeidup(3) = nodeid
  nodeiddn(3) = nodeid

#else
  ipsite(1)=mod(nodeid,NDIMX)
  ipsite(2)=mod(nodeid/NDIMX,NDIMY)
  ipsite(3)=mod(nodeid/NDIMX/NDIMY,NDIMZ)

  ipeo=mod(ipsite(1)*NX+ipsite(2)*NY+ipsite(3)*NZ,2)

  nodeidup(1)=mod(ipsite(1)+1,NDIMX)              &
                 +ipsite(2)               *NDIMX  &
                 +ipsite(3)               *NDIMX*NDIMY
  nodeiddn(1)=mod(ipsite(1)-1+NDIMX,NDIMX)        &
                 +ipsite(2)               *NDIMX  &
                 +ipsite(3)               *NDIMX*NDIMY
  nodeidup(2)=    ipsite(1)                       &
             +mod(ipsite(2)+1,NDIMY)      *NDIMX  &
                 +ipsite(3)               *NDIMX*NDIMY
  nodeiddn(2)=    ipsite(1)                       &
             +mod(ipsite(2)-1+NDIMY,NDIMY)*NDIMX  &
                 +ipsite(3)               *NDIMX*NDIMY
  nodeidup(3)=    ipsite(1)                       &
                 +ipsite(2)               *NDIMX  &
             +mod(ipsite(3)+1,NDIMZ)      *NDIMX*NDIMY
  nodeiddn(3)=    ipsite(1)                       &
                 +ipsite(2)               *NDIMX  &
             +mod(ipsite(3)-1+NDIMZ,NDIMZ)*NDIMX*NDIMY
#endif

#else
  ipsite(1)=0
  ipsite(2)=0
  ipsite(3)=0
  ipeo=0
  nodeid=0
#endif

  m_is_initialized = .TRUE.

  return
end subroutine

subroutine delete_lattice(this)
!
! Delete lattice constants, node information (next to nearest node rank etc.)
!
  use comlib
  implicit none
  type(lattice_world), intent(inout) :: this
  if (.not.m_is_initialized) return

#ifndef _singlePU
  call comlib_finalize()
#endif

  m_is_initialized = .FALSE.
  return
end subroutine


subroutine print_lattice(this)
!
! Print out Lattice configuration (size,partition,etc...) on display
!
  implicit none
  type(lattice_world), intent(inout) :: this
  if (nodeid==0) then
    write(*,'(10X,"   Lattice Size(XYZT) :",4I3)')NTX,NTY,NTZ,NTT
    write(*,'(10X,"PU Lattice Size(XYZT) :",4I3)')NX ,NY ,NZ ,NT
    write(*,'(10X,"  PU Array Size(XYZT) :",4I3)')NDIMX,NDIMY,NDIMZ,1
  endif
  return
end subroutine

function is_initialized_lattice(this) result(status)
!
! return lattice initialization status
!
!  .true.:: lattice is initialized
! .false.:: lattice is not initialized
!
  implicit none
  type(lattice_world), intent(in) :: this
  logical :: status 
  status = m_is_initialized
  return
end function

end module
