#include <config.h>

module onfly_measure_class
  use comlib
  use lattice_class
  use error_class
  use hmc_status_class
  use field_gauge_class
!--- measurement classes
  use ploop_class
!!$  use sf_measure_class

  implicit none
  private

  public :: onfly_measurement
  public :: new
  public :: delete
  public :: measure

  type onfly_measurement
    ! Measurement of Polyakov loops
    type(ploop_measurement), pointer :: ploop_obj => NULL()
!!$    ! Measurement of SF coupling constant
!!$    type(sf_measurement), pointer :: sf_obj => NULL()
  end type onfly_measurement

  interface new
    module procedure new_onfly_measurement
  end interface new

  interface delete
    module procedure delete_onfly_measurement
  end interface delete

  interface measure
    module procedure measure_onfly_measurement
  end interface

contains
subroutine new_onfly_measurement( this, infname, status_hmc )
  use comlib, only : comlib_bcast
  use file_tools_class
  implicit none
  type(onfly_measurement), intent(inout) :: this
  character(LEN=*), intent(in) :: infname
  type(hmc_status), intent(in) :: status_hmc
  !-------------------------------------------------------
  integer :: unit_id
  character(LEN=CHARLEN) :: str
  integer :: flag_ploop
  character(LEN=CHARLEN) :: fname_ploop
  integer :: flag_sf
  character(LEN=CHARLEN) :: fname_sf
 
  unit_id = search_free_file_unit()

  if ( nodeid == 0 ) then
    open(unit_id,file=TRIM(infname),status='old',form='formatted')

    ! Polyakov loop
    read(unit_id,*) str ! comment line
    read(unit_id,*) flag_ploop
    read(unit_id,'(A)') fname_ploop ! job file name

!!$   ! SF coupling constant
!!$    read(unit_id,*) str ! comment line
!!$    read(unit_id,*) flag_sf
!!$    read(unit_id,'(A)') fname_sf ! job file name

    close(unit_id)
  endif

  ! Polyakov loop
#ifndef _singlePU
  call comlib_bcast( flag_ploop, 0 )
  call comlib_bcast( fname_ploop, 0 )
#endif
  if ( flag_ploop /= 0 ) then
    allocate( this%ploop_obj )
    call new( this%ploop_obj, TRIM(fname_ploop), status_hmc )
  endif

!!$  ! SF coupling constant
!!$#ifndef _singlePU
!!$  call comlib_bcast( flag_sf, 0 )
!!$  call comlib_bcast( fname_sf, 0 )
!!$#endif
!!$  if ( flag_sf /= 0 ) then
!!$    allocate( this%sf_obj )
!!$    call new( this%sf_obj, TRIM(fname_sf), status_hmc )
!!$  endif

  return
end subroutine new_onfly_measurement


subroutine delete_onfly_measurement( this )
  implicit none
  type(onfly_measurement), intent(inout) :: this

  if ( associated( this%ploop_obj ) ) then
    call delete( this%ploop_obj)
    deallocate( this%ploop_obj )
  endif
  this%ploop_obj => NULL()
!!$  if ( associated( this%sf_obj ) ) then
!!$    call delete( this%sf_obj)
!!$    deallocate( this%sf_obj )
!!$  endif
!!$  this%sf_obj => NULL()

  if (0==nodeid) then
    write(*,'(80("="))')
  endif

  return
end subroutine delete_onfly_measurement


subroutine measure_onfly_measurement( this, u )
  implicit none
  type(onfly_measurement), intent(inout) :: this
  type(vfield_gluon_wg), intent(in) :: u

  if ( associated( this%ploop_obj ) ) call measure( this%ploop_obj, u )
!!$  if ( associated( this%sf_obj ) ) call measure( this%sf_obj, u )

  return
end subroutine measure_onfly_measurement

end module onfly_measure_class
