#include <config.h>

program main_hadron
!
! = Hadron propagator solver (includes quark prop solver)
!
  use comlib
  use file_tools_class
  use error_class
  use lattice_class
  use field_gauge_class
  use timer_class
  use wavefunc_class
  use quark_prop_class
  use hadron_class
  use main_hadron_class
#ifdef _STOUT
  use hmc_stout_class
#endif
  implicit none
  character(len=CHARLEN), parameter :: &
 &   prog_ver="$Id: main_hadron.F90,v 1.1 2006/10/04 01:52:25 ishikawa Exp $"

  type(main_hadron_obj) :: main

  type(meson_prop_obj)      :: meson
  type(meson_mom_prop_obj)  :: meson_mom
  type(baryon_prop_obj)     :: baryon
  type(hadrons_obj)         :: hadrons

  type(quark_prop), allocatable :: qprop(:,:)
  type(snk_wavefunc_obj), allocatable, target :: snk_wf
#ifdef _STOUT
  type(stout_smearing_parameters) :: stout
#endif


  integer :: iout
  integer :: traj,mu
  real(DP) :: plq0
  character(len=CHARLEN)    :: char
  integer :: isrc,iqk,isrc1,isrc2,isrcd,iflv
  integer :: ifm,ifb

  integer :: nfl,nfh,nf,iqk1,iqk2

  call new(main)

  if (nodeid == 0) then
    write(*,'(80("-"))')
    write(*,'(1X,A," for ",A)')TRIM(prog_ver),TRIM(_VERSION_)
    write(*,'(80("-"))')
  endif

  call new(hadrons)

  iout = search_free_file_unit()
  call read(main,iout)
  close(iout)
  call print(main)

  allocate(snk_wf)
  call new(snk_wf)
  snk_wf%param = main%wavefunc(1)%param
  call get(snk_wf)


  allocate(qprop(2,2))
  do isrc=1,2
  do iqk=1,2
    call new(qprop(iqk,isrc))
  enddo
  enddo

  do traj=main%ini_traj,main%end_traj,main%traj_skip

    !-----------------------
    ! read config
    !-----------------------
    call read(traj,main%config_path,main%config_name, &
 &                 main%config_prof,main%u)

#ifdef _STOUT
    if (main%switch_stout == SW_ON) then
      call new(stout,alpha=main%stout_alpha,nstout=main%stout_nstep,is_smear_only=.TRUE.)
      call print(stout)
      call smear(stout,main%u)
      call assign(main%u,stout%usmr(1))
      call delete(stout)
      call copy_boundary(main%u)
      call plaquette(main%u,plq0)
      if (nodeid==0) write(*,'(" PLQ(STOUT):",F24.16)')plq0
    endif
#endif

    !-----------------------
    ! create hadron props
    !-----------------------
    meson%itraj           =  traj
    meson%path            =  main%hadron_path
    meson%config_prof     => main%config_prof
    meson_mom%itraj       =  traj
    meson_mom%path        =  main%hadron_path
    meson_mom%config_prof => main%config_prof
    baryon%itraj          =  traj
    baryon%path           =  main%hadron_path
    baryon%config_prof    => main%config_prof

    call new(meson)
    call new(meson_mom)
    call new(baryon)


    !-----------------------
    ! create quark props
    !-----------------------
    do isrc = 1,2
    do iqk  = 1,2
      qprop(iqk,isrc)%parameter     => main%quark_parameter(iqk)

      select case(main%num_wave)
      case (3) ! independent strange quark wavefunction and up/down quark wavefunction
        if ( iqk == 1 ) then
          if ( isrc == 1 ) then
            qprop(iqk,isrc)%wavefunc_sous => main%wavefunc(1)
          else if ( isrc == 2 ) then
            qprop(iqk,isrc)%wavefunc_sous => main%wavefunc(2)
          endif
        else if ( iqk == 2 ) then
          if ( isrc == 1 ) then
            qprop(iqk,isrc)%wavefunc_sous => main%wavefunc(1)
          else if ( isrc == 2 ) then
            qprop(iqk,isrc)%wavefunc_sous => main%wavefunc(3) ! strange quark smeared wavefunction
          endif
        endif
      case (2)
        qprop(iqk,isrc)%wavefunc_sous => main%wavefunc(isrc)
      end select

      qprop(iqk,isrc)%smear_sous    =  isrc-1

      qprop(iqk,isrc)%wavefunc_sink => snk_wf
      qprop(iqk,isrc)%smear_sink    =  0

      if (nodeid==0) then
        write(*,'("")')
        write(*,'(80("="))')
        write(*,'("Begin ==== Quark #",I3," Source #",I3)')iqk,isrc
      endif
      call print(qprop(iqk,isrc)%parameter)
      if (nodeid==0) then
        write(*,'("")')
      endif

      !===============================
      ! generate source wavefunction
      !===============================
      call print(qprop(iqk,isrc)%wavefunc_sous)
      call   get(qprop(iqk,isrc)%wavefunc_sous)

      !===================================================
      ! compute smared source-local sink quark propagator
      !===================================================
      call set_source(qprop(iqk,isrc))
      call get_prop(qprop(iqk,isrc),main%u)
      deallocate(qprop(iqk,isrc)%wavefunc_sous%wavefunc)


      if (nodeid==0) then
        write(*,'("")')
      endif
      call print_statistics(qprop(iqk,isrc))
      if (nodeid==0) then
        write(*,'("End ==== Quark #",I3," Source #",I3)')iqk,isrc
        write(*,'("")')
      endif

    enddo
    enddo


    nfl=1
    nfh=1
    nf=nfl+nfh
    
    ifm=1
    ifb=1

    !---------------------------
    ! Light-Light flavor loop
    !---------------------------
    do iqk = 1,nfl
    do isrc1 = 1,2
    do isrcd = 1,2
      isrc2=mod(isrc1-1+isrcd-1,2)+1

      !---------------------------
      ! Light-Light mesons
      !---------------------------
      call get_prop(meson,    qprop(iqk,isrc1),qprop(iqk,isrc2),main%u)
      call get_prop(meson_mom,qprop(iqk,isrc1),qprop(iqk,isrc2))
      call save(meson,ifm)
      call save(meson_mom,ifm)
      ifm=ifm+1

      !------------------------------
      ! Light-Light-Light baryons
      !------------------------------
      if (isrc1 == isrc2) then
        call get_prop(baryon,qprop(iqk,isrc1))
      else
        call get_prop(baryon,qprop(iqk,isrc1),qprop(iqk,isrc2))
      endif
      call save(baryon,ifb)
      ifb=ifb+1
    enddo
    enddo
    enddo

    !------------------------------
    ! Light-Strange flavor loop
    !------------------------------
    do iqk1  = 1,nfl
    do iqk2  = nfl+1,nfl+nfh
    do isrc1 = 1,2
    do isrcd = 1,2
      isrc2=mod(isrc1-1+isrcd-1,2)+1

      !------------------------------
      ! Light-Strange mesons
      !------------------------------
      call get_prop(meson,    qprop(iqk1,isrc1),qprop(iqk2,isrc2),main%u)
      call get_prop(meson_mom,qprop(iqk1,isrc1),qprop(iqk2,isrc2))
      call save(meson,ifm)
      call save(meson_mom,ifm)
      ifm=ifm+1

      !---------------------------------
      ! Light-Light-Strange baryons
      !---------------------------------
      call get_prop(baryon,qprop(iqk1,isrc1),qprop(iqk2,isrc2))
      call save(baryon,ifb)
      ifb=ifb+1

      !---------------------------------
      ! Strange-Strange-Light baryons
      !---------------------------------
      call get_prop(baryon,qprop(iqk2,isrc2),qprop(iqk1,isrc1))
      call save(baryon,ifb)
      ifb=ifb+1

    enddo
    enddo
    enddo
    enddo

    !-----------------------------
    ! Strange-Strange loop
    !-----------------------------
    do iqk=nfl+1,nfl+nfh
    do isrc1=1,2
    do isrcd=1,2
      isrc2=mod(isrc1-1+isrcd-1,2)+1

      !--------------------------------
      ! Strange-Strange mesons
      !--------------------------------
      call get_prop(meson,    qprop(iqk,isrc1),qprop(iqk,isrc2),main%u)
      call get_prop(meson_mom,qprop(iqk,isrc1),qprop(iqk,isrc2))
      call save(meson,ifm)
      call save(meson_mom,ifm)
      ifm=ifm+1

      !-----------------------------------
      ! Strange-Strange-Strange baryons
      !-----------------------------------
      if (isrc1 == isrc2) then
        call get_prop(baryon,qprop(iqk,isrc1))
      else
        call get_prop(baryon,qprop(iqk,isrc1),qprop(iqk,isrc2))
      endif
      call save(baryon,ifb)
      ifb=ifb+1

    enddo
    enddo
    enddo

    call delete(meson)
    call delete(meson_mom)
    call delete(baryon)

  enddo

  do isrc = 1,2
  do iqk  = 1,2
    call delete(qprop(iqk,isrc))
  enddo
  enddo
  deallocate(qprop)

  call delete(main)

  stop
end program
